/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-01 15:17:10.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.config;

import cn.com.qmth.examcloud.app.model.*;
import cn.com.qmth.examcloud.app.service.CoreAuthService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;

import javax.servlet.FilterConfig;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;

import static cn.com.qmth.examcloud.app.model.Constants.PLATFORM_SESSION_EXPIRE_TIME;

/**
 * 认证信息过滤器
 *
 * @author: fengdesheng
 * @since: 2018/8/1
 */
public class TokenFilter implements Filter {

    private final static Logger log = LoggerFactory.getLogger(TokenFilter.class);

    private CoreAuthService authService;

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        log.debug("init");
    }

    @Override
    public void destroy() {
        log.debug("destroy");
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;

        //移动端设备请求必须带“设备编号”参数
        /*String deviceId = request.getHeader(Constants.PARAM_DEVICE_ID);
        if (StringUtils.isBlank(deviceId)) {
            Result result = new Result().error("[APP] deviceId must be not empty.");
            this.render(response, result.toString(), true);
            return;
        }*/

        Continue reqContinue = new Continue();
        CustomHttpServletRequest customRequest = this.initCustomRequest(request, response, reqContinue);
        if (customRequest != null) {
            filterChain.doFilter(customRequest, servletResponse);
            return;
        }

        if (reqContinue.yes) {
            filterChain.doFilter(servletRequest, servletResponse);
        }
    }

    private CustomHttpServletRequest initCustomRequest(HttpServletRequest request, HttpServletResponse response, Continue reqContinue) {
        String url = request.getServletPath();
        if (url.endsWith("/user/login") || url.endsWith("/user/login/verify")) {
            //处理登录接口
            String rootOrgIdStr = request.getParameter(Constants.PARAM_ROOT_ORG_ID);
            String accountType = request.getParameter(Constants.PARAM_ACCOUNT_TYPE);
            String account = request.getParameter(Constants.PARAM_ACCOUNT);

            Long rootOrgId = null;
            try {
                rootOrgId = Long.parseLong(rootOrgIdStr);
            } catch (NumberFormatException e) {
                //ignore
            }

            if (rootOrgId == null) {
                //rootOrgId = this.loadRootOrgId(request);
                rootOrgId = authService.getRootOrgIdBySecurityPhone(account);
            }

            if (rootOrgId == null) {
                reqContinue.yes = false;
                this.renderError(response, new Result().error("学校ID获取失败！").toString());
                return null;
            }

            boolean isOpen = authService.isOpenApp(rootOrgId);
            log.info("[Check Open APP] Result is " + isOpen);
            if (!isOpen) {
                reqContinue.yes = false;
                this.renderError(response, new Result().error("当前学校尚未开放APP功能！").toString());
                return null;
            }

            boolean isDoing = authService.isDoingExam(rootOrgId, accountType, account);
            log.info("[Check Doing Exam] Result is " + isDoing);
            if (isDoing) {
                reqContinue.yes = false;
                this.renderError(response, new Result().error("尚在考试中不允许登录！").toString());
                return null;
            }
        }

        String appKey = request.getHeader(Constants.PARAM_APP_KEY);
        String appToken = request.getHeader(Constants.PARAM_APP_TOKEN);
        if (StringUtils.isBlank(appKey) || StringUtils.isBlank(appToken)) {
            //key,token为空，则不用处理
            return null;
        }

        //通过key获取原始登录信息
        LoginInfo loginInfo = authService.getLoginInfo(appKey);
        if (loginInfo == null) {
            //原始登录信息为空，则代表尚未登录不用处理
            return null;
        }

        //同一个账号不同移动端设备，支持互踢
        if (!appToken.equals(loginInfo.getAppToken())) {
            //App Token参数值无效
            return null;
        }

        //处理已登录信息
        CustomHttpServletRequest customRequest = new CustomHttpServletRequest(request);
        boolean isAllow = this.filterAccessUrl(url);
        if (!isAllow) {
            if (loginInfo.hasExpired(PLATFORM_SESSION_EXPIRE_TIME)) {
                if (StringUtils.isBlank(loginInfo.getPassword())) {
                    log.info("Can't reLogin, no password.");
                    return null;
                }

                boolean isDoing = authService.isDoingExam(loginInfo.getRootOrgId(), loginInfo.getAccountType(), loginInfo.getAccount());
                log.info("[Check Doing Exam] result is " + isDoing);
                if (isDoing) {
                    reqContinue.yes = false;
                    this.renderError(response, new Result().error("尚在考试中不允许登录！").toString());
                    return null;
                }

                //判断原始登录Token是否在有效时间内，否则自动登录续期
                this.reLogin(loginInfo);
            }
        }

        //将App Token替换为原始登录Token到request请求中
        customRequest.addHeader(Constants.PARAM_APP_KEY, loginInfo.getKey());
        customRequest.addHeader(Constants.PARAM_APP_TOKEN, loginInfo.getToken());

        //初始化内部接口请求鉴权
        authService.initRequestTrace();
        return customRequest;
    }

    private Long loadRootOrgId(HttpServletRequest request) {
        String accountType;
        if (request.getServletPath().endsWith("/user/login/verify")) {
            accountType = LoginType.STUDENT_PHONE.name();
        } else {
            accountType = request.getParameter("accountType");
        }

        String account = request.getParameter("account");
        String password = request.getParameter("password");
        String smsCode = request.getParameter("smsCode");
        String domain = request.getParameter("domain");

        LoginInfo loginInfo = new LoginInfo();
        loginInfo.setNoSession(true);
        loginInfo.setAccountType(accountType);
        loginInfo.setAccount(account);
        loginInfo.setPassword(password);
        loginInfo.setSmsCode(smsCode);
        loginInfo.setDomain(domain);

        Result<UserInfo> result = authService.login(loginInfo);
        if (result.isSuccess() && result.getData() != null) {
            return result.getData().getRootOrgId();
        }
        return null;
    }

    private void reLogin(LoginInfo loginInfo) {
        try {
            Result<UserInfo> result = authService.login(loginInfo);
            UserInfo userInfo = result.getData();
            if (userInfo == null) {
                log.debug("reLogin failed,Maybe user's password has changed.");
                return;
            }
            //登录成功后缓存新的Token信息
            log.info(String.format("key:%s newToken:%s", userInfo.getKey(), userInfo.getToken()));
            loginInfo.setUserName(userInfo.getDisplayName());
            loginInfo.setToken(userInfo.getToken());
            loginInfo.setCreateTime(new Date());
            authService.cacheLoginInfo(loginInfo, userInfo.getKey());
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    private boolean filterAccessUrl(String url) {
        //过滤不用处理的访问地址
        if (url.contains("/user/logout")) {
            return true;
        }
        return false;
    }

    private void renderError(HttpServletResponse response, String json) {
        this.render(response, json, false);
    }

    private void render(HttpServletResponse response, String json, boolean isOk) {
        PrintWriter out = null;
        try {
            response.setStatus(isOk ? HttpStatus.OK.value() : HttpStatus.INTERNAL_SERVER_ERROR.value());
            response.setCharacterEncoding("UTF-8");
            response.setContentType("application/json;charset=UTF-8");
            out = response.getWriter();
            out.print(json);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }

    public void setAuthService(CoreAuthService authService) {
        this.authService = authService;
    }

    class Continue {

        boolean yes = true;
    }

}