/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-17 17:32:00.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.utils;

import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.ResBody;
import cn.com.qmth.examcloud.app.model.Result;
import com.alibaba.fastjson.JSON;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.ConnectException;
import java.util.UUID;

import static cn.com.qmth.examcloud.app.model.Constants.*;

/**
 * @author: fengdesheng
 * @since: 2018/7/16
 */
public class HttpUtils {
    private static Logger log = LoggerFactory.getLogger(HttpUtils.class);

    public static String getRandomTraceId() {
        String traceId = ThreadUtils.getTraceID();
        if (StringUtils.isBlank(traceId)) {
            traceId = UUID.randomUUID().toString().replace("-", "");
        }
        return traceId;
    }

    public static Result<String> doGet(String requestUrl, String key, String token) throws Exception {
        String traceId = getRandomTraceId();

        //封装请求参数
        Request request = new Request.Builder()
                .url(requestUrl)
                .get()
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .addHeader(PARAM_TRACE_ID, traceId)
                .addHeader(PARAM_CLIENT, PARAM_CLIENT_VALUE)
                .build();

        //执行请求
        log.info("[GET] " + requestUrl);
        return call(request, traceId);
    }

    public static Result<String> doPost(String requestUrl, RequestBody formBody, String key, String token) throws Exception {
        String traceId = getRandomTraceId();

        //封装请求参数
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .addHeader(PARAM_TRACE_ID, traceId)
                .addHeader(PARAM_CLIENT, PARAM_CLIENT_VALUE)
                .build();

        //执行请求
        log.info("[POST] " + requestUrl);
        return call(request, traceId);
    }

    public static Result<String> doPut(String requestUrl, RequestBody formBody, String key, String token) throws Exception {
        String traceId = getRandomTraceId();

        //封装请求参数
        Request request = new Request.Builder()
                .url(requestUrl)
                .put(formBody)
                .addHeader(PARAM_KEY, key)
                .addHeader(PARAM_TOKEN, token)
                .addHeader(PARAM_TRACE_ID, traceId)
                .addHeader(PARAM_CLIENT, PARAM_CLIENT_VALUE)
                .build();

        //执行请求
        log.info("[PUT] " + requestUrl);
        return call(request, traceId);
    }

    public static Result<String> call(Request request, String traceId) throws Exception {
        Response response;
        try {
            response = HttpClientBuilder.getClient().newCall(request).execute();
        } catch (ConnectException e) {
            log.error("[ConnectException] " + e.getMessage());
            return new Result<>().error("服务访问失败！");
        }

        String bodyStr = response.body().string();
        if (response.isSuccessful()) {
            log.info("[Response] is success!");
            return new Result().success(bodyStr);
        } else {
            log.warn("[Response] is fail! body:" + bodyStr);
            ResBody body = new JsonMapper().fromJson(bodyStr, ResBody.class);
            if (body != null && body.getCode() != null) {
                if (Constants.CODE_403.equals(body.getCode())) {
                    return new Result().noAuthError();
                }
                if (Constants.CODE_404.equals(body.getCode())) {
                    return new Result().noFoundError();
                }
                if (Constants.CODE_0.equals(body.getCode())) {
                    return new Result().error(body.getErrorMsg());
                }
                if (Constants.CODE_P001012.equals(body.getCode())) {
                    return new Result().noAuthError();
                }
                if (Constants.CODE_B001012.equals(body.getCode())) {
                    return new Result().noAuthError();
                }
                return new Result().error(body.getDesc());
            }
            return new Result().error(bodyStr);
        }
    }

    /**
     * 如果是JSON字符串，则过滤掉为空的属性
     */
    public static String filterNullAttributes(String str) {
        if (StringUtils.isBlank(str)) {
            return "";
        }
        try {
            Object obj = JSON.parse(str);
            return obj.toString();
        } catch (Exception e) {
            return str;
        }
    }

    /**
     * 替换字符串中所有的图片标签
     */
    public static String replaceImages(String html) {
        if (html == null) {
            return null;
        }
        String reg = "<\\s*img\\s+([^>]*)\\s*>";
        return html.replaceAll(reg, "");
    }

    /*
    public static void main(String[] args) {
        String json = "{\"id\":1,\"answer\":null,\"question\":{\"id\":2,\"answer\":null,\"question\":{\"id\":3,\"answer\":null,\"question\":{\"id\":4,\"answer\":null,,\"question\":{\"id\":5,\"answer\":null}}}}}";
        String result = HttpUtils.filterNullAttributes(json);
        System.out.println(result);
        String html = "111< img src='' >222< img / >333";
        System.out.println(replaceImages(html));
    }
    */

}