/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.SysProperty;
import cn.com.qmth.examcloud.app.core.exception.ApiException;
import cn.com.qmth.examcloud.app.core.utils.HttpClientBuilder;
import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.core.utils.JsonMapper;
import cn.com.qmth.examcloud.app.core.utils.ThreadUtils;
import cn.com.qmth.examcloud.app.model.*;
import cn.com.qmth.examcloud.app.service.CoreAuthService;
import cn.com.qmth.examcloud.app.service.RedisService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.OrgPropertyCacheBean;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.*;

/**
 * 认证中心业务服务接口
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
@Service
public class CoreAuthServiceImpl implements CoreAuthService {

    private static Logger log = LoggerFactory.getLogger(CoreAuthServiceImpl.class);

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private RedisService redisService;

    @Autowired
    private StudentCloudService studentCloudService;

    @Override
    public boolean isDoingExam(Long rootOrgId, String accountType, String account) {
        if (rootOrgId == null) {
            log.warn("[Check Doing Exam] rootOrgId is null");
            return false;
        }

        log.info(String.format("[Check Doing Exam] rootOrgId=%s, account=%s, accountType=%s", rootOrgId, account, accountType));

        Map<String, String> params = new HashMap<>();
        params.put("rootOrgId", rootOrgId.toString());
        if (LoginType.STUDENT_CODE.name().equals(accountType)) {
            params.put("studentCode", account);
        } else if (LoginType.STUDENT_IDENTITY_NUMBER.name().equals(accountType)) {
            params.put("identityNumber", account);
        } else {
            //LoginType.STUDENT_PHONE
            params.put("phone", account);
        }

        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/getStudentOnLineExamInfo", sysProperty.getCoreOeStudentUrl());
        try {
            //执行请求
            String json = new JsonMapper().toJson(params);
            RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
            Request request = new Request.Builder()
                    .post(formBody)
                    .url(requestUrl)
                    .build();
            Response response = HttpClientBuilder.getClient().newCall(request).execute();

            String bodyStr = response.body().string();
            if (response.isSuccessful()) {
                ExamInfo info = new JsonMapper().fromJson(bodyStr, ExamInfo.class);
                if (info != null) {
                    return info.getExistExamingRecord();
                }
            } else {
                log.warn(bodyStr);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

        log.warn("[Check Doing Exam] fail");
        return false;
    }

    @Override
    public boolean isOpenApp(Long rootOrgId) {
        if (rootOrgId == null) {
            log.warn("[Check Open APP] rootOrgId is null");
            return false;
        }

        try {
            OrgPropertyCacheBean property = CacheHelper.getOrgProperty(rootOrgId, "APP_ENABLED");

            if ("true".equals(property.getValue())) {
                return true;
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return false;
    }

    @Override
    public Long getRootOrgIdBySecurityPhone(String securityPhone) {
        GetStudentReq req = new GetStudentReq();
        req.setSecurityPhone(securityPhone);

        try {
            GetStudentResp resp = studentCloudService.getStudent(req);
            return resp.getStudentInfo().getRootOrgId();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

        return null;
    }

    @Override
    public Result<UserInfo> login(LoginInfo loginInfo) {
        Assert.notNull(loginInfo, "LoginInfo must be not null.");
        if (StringUtils.isBlank(loginInfo.getAccountType())) {
            loginInfo.setAccountType(LoginType.STUDENT_PHONE.name());
        }

        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/login", sysProperty.getCoreBasicUrl());
        Map<String, String> params = new HashMap<>();
        params.put("accountValue", loginInfo.getAccount());
        params.put("password", loginInfo.getPassword());
        params.put("accountType", loginInfo.getAccountType());
        params.put("rootOrgId", loginInfo.getRootOrgId() != null ? loginInfo.getRootOrgId().toString() : "");
        params.put("domain", loginInfo.getDomain());
        params.put("smsCode", loginInfo.getSmsCode());
        if (loginInfo.getNoSession() != null && loginInfo.getNoSession()) {
            params.put("noSession", loginInfo.getNoSession().toString());
        }
        String json = new JsonMapper().toJson(params);

        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .build();

        //执行请求
        try (Response response = HttpClientBuilder.getClient().newCall(request).execute();) {
            String bodyStr = response.body().string();
            if (response.isSuccessful()) {
                //获取用户信息
                UserInfo userInfo = new JsonMapper().fromJson(bodyStr, UserInfo.class);
                return new Result().success(userInfo);
            } else {
                log.warn("Http response is " + bodyStr);
                ResBody body = new JsonMapper().fromJson(bodyStr, ResBody.class);
                if (body != null && body.getCode() != null) {
                    return new Result().error(body.getDesc());
                }
                return new Result().error(bodyStr);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new Result().error("登录异常！");
        }
    }

    @Override
    public Result logout(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/logout", sysProperty.getCoreAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add(PARAM_KEY, key)
                .add(PARAM_TOKEN, token)
                .build();

        //清除缓存用户登录信息
        this.removeCacheLoginInfo(key);

        //退出登录
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result updateStudentPassword(String key, String token, Long studentId, String password, String newPassword) throws Exception {
        Assert.notNull(studentId, "StudentId must be not null.");
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/student/password", sysProperty.getCoreAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("studentId", studentId.toString())
                .add("password", password)
                .add("newPassword", newPassword)
                .build();
        return HttpUtils.doPut(requestUrl, formBody, key, token);
    }

    @Override
    public Result resetStudentPassword(String key, String token, String newPassword) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/student/password/direct", sysProperty.getCoreAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("newPassword", newPassword)
                .build();
        return HttpUtils.doPut(requestUrl, formBody, key, token);
    }

    @Override
    public Result sendSmsCode(String key, String token, String phone) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/sendVerificationCode", sysProperty.getCoreAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("phone", phone)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result code4Student(String phone, Boolean bound) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/sendVerificationCode4Student", sysProperty.getCoreAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("phone", phone)
                .add("bound", bound.toString())
                .build();
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .build();

        //执行请求
        String traceId = HttpUtils.getRandomTraceId();
        log.info("[TRACE_ID] " + traceId + " [POST] " + requestUrl);
        return HttpUtils.call(request, traceId);
    }

    @Override
    public Result userBindingPhone(String key, String token, String phone, String code) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/bindSecurityPhone", sysProperty.getCoreAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("phone", phone)
                .add("verificationCode", code)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    /**
     * 缓存用户登录信息
     */
    @Override
    public void cacheLoginInfo(LoginInfo loginInfo, String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("key must be not empty.");
        }
        String jsonStr = new JsonMapper().toJson(loginInfo);
        redisService.set(APP_SESSION_USER_KEY_PREFIX + key, jsonStr, APP_SESSION_EXPIRE_TIME);
    }

    /**
     * 清除缓存用户登录信息
     */
    private void removeCacheLoginInfo(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("key must be not empty.");
        }
        redisService.delete(APP_SESSION_USER_KEY_PREFIX + key);
    }

    /**
     * 获取缓存中的用户登录信息
     */
    @Override
    public LoginInfo getLoginInfo(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("key must be not empty.");
        }
        return redisService.fromJson(APP_SESSION_USER_KEY_PREFIX + key, LoginInfo.class);
    }

    /**
     * 获取平台端的默认过期时间(秒)
     */
    @Override
    public int getSessionTimeout() {
        try {
            String timeout = redisService.get(Constants.PLATFORM_SESSION_TIMEOUT_KEY);
            if (StringUtils.isNotEmpty(timeout)) {
                return Integer.parseInt(timeout);
            }
        } catch (Exception e) {
            //ignore
        }
        return Constants.PLATFORM_SESSION_EXPIRE_TIME;
    }

    /**
     * 初始化内部接口请求鉴权
     */
    @Override
    public void initRequestTrace() {
        String key = "C_" + ThreadUtils.getTraceID();
        Long millis = System.currentTimeMillis();
        redisService.set(key, millis.toString(), 15);
    }

}